@@ -20,6 +20,8 @@ c.GraderService.service_host = "127.0.0.1"
 service_dir = os.path.join(os.getcwd(), "service_dir")
 c.GraderService.grader_service_dir = service_dir
 
+c.GraderService.log_level = "DEBUG"
+
 c.RequestHandlerConfig.autograde_executor_class = LocalAutogradeExecutor
 
 c.CeleryApp.conf = dict(
@@ -50,7 +52,9 @@ c.GraderService.authenticator_class = DummyAuthenticator
 c.Authenticator.allowed_users = {"admin", "instructor", "student", "tutor"}
 c.Authenticator.admin_users = {"admin"}
 c.GraderService.load_roles = {
-    "lect1": {"members": ["admin", "instructor"], "role": "instructor"},
-    "lect1": {"members": ["tutor"], "role": "tutor"},
-    "lect1": {"members": ["student"], "role": "student"},
+    "lecture1": [
+        {"members": ["student"], "role": "student"},
+        {"members": ["tutor"], "role": "tutor"},
+        {"members": ["instructor", "admin"], "role": "instructor"},
+    ]
 }
