@@ -3,8 +3,11 @@
 from grader_service.api.models.assignment import Assignment
 from grader_service.api.models.assignment_detail import AssignmentDetail
 from grader_service.api.models.assignment_settings import AssignmentSettings
+from grader_service.api.models.auto_status import AutoStatus
 from grader_service.api.models.error_message import ErrorMessage
+from grader_service.api.models.feedback_status import FeedbackStatus
 from grader_service.api.models.lecture import Lecture
+from grader_service.api.models.manual_status import ManualStatus
 from grader_service.api.models.remote_file_status import RemoteFileStatus
 from grader_service.api.models.submission import Submission
 from grader_service.api.models.submission_period import SubmissionPeriod
