@@ -12,7 +12,6 @@ import os
 import shlex
 import shutil
 import subprocess
-from dataclasses import dataclass
 from datetime import datetime
 from pathlib import Path
 from subprocess import CalledProcessError
@@ -28,18 +27,11 @@ from grader_service.convert.converters.autograde import Autograde
 from grader_service.convert.gradebook.models import GradeBookModel
 from grader_service.orm.assignment import Assignment
 from grader_service.orm.lecture import Lecture
-from grader_service.orm.submission import Submission
+from grader_service.orm.submission import AutoStatus, ManualStatus, Submission
 from grader_service.orm.submission_logs import SubmissionLogs
 from grader_service.orm.submission_properties import SubmissionProperties
 
 
-@dataclass
-class AutogradingStatus:
-    status: str
-    started_at: datetime
-    finished_at: datetime
-
-
 def default_timeout_func(lecture: Lecture) -> int:
     return 360
 
@@ -102,8 +94,9 @@ class LocalAutogradeExecutor(LoggingConfigurable):
         It re-raises all exceptions that happen while running.
         """
         self.log.info(
-            f"Starting autograding job for submission "
-            f"{self.submission.id} in {self.__class__.__name__}"
+            "Starting autograding job for submission %s in %s",
+            self.submission.id,
+            self.__class__.__name__,
         )
         try:
             self._pull_submission()
@@ -113,19 +106,23 @@ class LocalAutogradeExecutor(LoggingConfigurable):
             self._set_properties()
             self._push_results()
             self._set_db_state()
-            ts = round((self.autograding_finished - self.autograding_start).total_seconds())
-            self.log.info(
-                f"Successfully completed autograding job for submission "
-                f"{self.submission.id} in {self.__class__.__name__};"
-                + f" took {ts // 60}min {ts % 60}s"
-            )
         except Exception:
             self.log.error(
-                f"Failed autograding job for submission "
-                f"{self.submission.id} in {self.__class__.__name__}",
+                "Failed autograding job for submission %s in %s",
+                self.submission.id,
+                self.__class__.__name__,
                 exc_info=True,
             )
             self._set_db_state(success=False)
+        else:
+            ts = round((self.autograding_finished - self.autograding_start).total_seconds())
+            self.log.info(
+                "Successfully completed autograding job for submission %s in %s; took %s min %s s",
+                self.submission.id,
+                self.__class__.__name__,
+                ts // 60,
+                ts % 60,
+            )
         finally:
             self._cleanup()
 
@@ -167,7 +164,7 @@ class LocalAutogradeExecutor(LoggingConfigurable):
 
         assignment: Assignment = self.submission.assignment
         lecture: Lecture = assignment.lecture
-        repo_name = self.submission.username
+        repo_name = self.submission.user.name
 
         if self.submission.edited:
             git_repo_path = os.path.join(
@@ -252,7 +249,7 @@ class LocalAutogradeExecutor(LoggingConfigurable):
         Checks if assignment was already graded and returns updated properties.
         :return: str
         """
-        if self.submission.manual_status == "not_graded":
+        if self.submission.manual_status == ManualStatus.NOT_GRADED:
             return self.assignment.properties
 
         assignment_properties = json.loads(self.assignment.properties)
@@ -281,14 +278,13 @@ class LocalAutogradeExecutor(LoggingConfigurable):
         Pushes the results to the autograde repository
         as a separate branch named after the commit hash of the submission.
         Removes the gradebook.json file before doing so.
-        :return: Coroutine
         """
         os.unlink(os.path.join(self.output_path, "gradebook.json"))
         self.log.info(f"Pushing files: {os.listdir(self.output_path)}")
 
         assignment: Assignment = self.submission.assignment
         lecture: Lecture = assignment.lecture
-        repo_name = self.submission.username
+        repo_name = self.submission.user.name
 
         git_repo_path = os.path.join(
             self.grader_service_dir,
@@ -428,9 +424,9 @@ class LocalAutogradeExecutor(LoggingConfigurable):
         :return: None
         """
         if success:
-            self.submission.auto_status = "automatically_graded"
+            self.submission.auto_status = AutoStatus.AUTOMATICALLY_GRADED
         else:
-            self.submission.auto_status = "grading_failed"
+            self.submission.auto_status = AutoStatus.GRADING_FAILED
 
         if self.grading_logs is not None:
             self.grading_logs = self.grading_logs.replace("\x00", "")
