@@ -33,17 +33,22 @@ from grader_service.handlers.handler_utils import GitRepoType, parse_ids
 from grader_service.orm.assignment import Assignment
 from grader_service.orm.base import DeleteState
 from grader_service.orm.lecture import Lecture
-from grader_service.orm.submission import Submission
+from grader_service.orm.submission import AutoStatus, FeedbackStatus, ManualStatus, Submission
 from grader_service.orm.submission_logs import SubmissionLogs
 from grader_service.orm.submission_properties import SubmissionProperties
 from grader_service.orm.takepart import Role, Scope
+from grader_service.orm.user import User
 from grader_service.plugins.lti import LTISyncGrades
 from grader_service.registry import VersionSpecifier, register_handler
 
+# Commit hash is used to differentiate between submissions created by instructors for students and
+# normal submissions by any user.
+INSTRUCTOR_SUBMISSION_COMMIT_CASH = "0" * 40
+
 
 def remove_points_from_submission(submissions):
     for s in submissions:
-        if s.feedback_status not in ("generated", "feedback_outdated"):
+        if s.feedback_status not in (FeedbackStatus.GENERATED, FeedbackStatus.FEEDBACK_OUTDATED):
             s.score = None
     return submissions
 
@@ -80,49 +85,43 @@ class SubmissionLectureHandler(GraderBaseHandler):
         if submission_filter not in ["latest", "best"]:
             raise HTTPError(
                 HTTPStatus.BAD_REQUEST,
-                reason="Filter parameter \
-            has to be either 'latest' or 'best'",
+                reason="Filter parameter has to be either 'latest' or 'best'",
             )
         response_format = self.get_argument("format", "json")
         if response_format not in ["json", "csv"]:
             raise HTTPError(
-                HTTPStatus.BAD_REQUEST,
-                reason="Response format \
-            can either be 'json' or 'csv'",
+                HTTPStatus.BAD_REQUEST, reason="Response format can either be 'json' or 'csv'"
             )
 
-        role: Role = self.get_role(lecture_id)
-        if role.role < Scope.tutor:
-            raise HTTPError(HTTPStatus.FORBIDDEN, reason="Forbidden")
-
         if submission_filter == "latest":
             subquery = (
-                self.session.query(Submission.username, func.max(Submission.date).label("max_date"))
+                self.session.query(Submission.user_id, func.max(Submission.date).label("max_date"))
                 .join(Assignment, Submission.assignid == Assignment.id)
                 .filter(Assignment.lectid == lecture_id)
                 .filter(Submission.deleted == DeleteState.active)
-                .group_by(Submission.username, Assignment.id)
+                .group_by(Submission.user_id, Assignment.id)
                 .subquery()
             )
         else:
             subquery = (
                 self.session.query(
-                    Submission.username, func.max(Submission.score).label("max_score")
+                    Submission.user_id, func.max(Submission.score).label("max_score")
                 )
                 .join(Assignment, Submission.assignid == Assignment.id)
                 .filter(Assignment.lectid == lecture_id)
                 .filter(Submission.deleted == DeleteState.active)
-                .group_by(Submission.username, Assignment.id)
+                .group_by(Submission.user_id, Assignment.id)
                 .subquery()
             )
 
         submissions_query = (
             self.session.query(
-                label("username", Submission.username),
+                label("username", User.name),
                 label("score", Submission.score),
                 label("assignment", Assignment.name),
             )
             .join(Assignment, Submission.assignid == Assignment.id)
+            .join(User, Submission.user_id == User.id)
             .filter(Assignment.lectid == lecture_id)
             .filter(Submission.deleted == DeleteState.active)
         )
@@ -131,7 +130,7 @@ class SubmissionLectureHandler(GraderBaseHandler):
             submissions = (
                 submissions_query.join(
                     subquery,
-                    (Submission.username == subquery.c.username)
+                    (Submission.user_id == subquery.c.user_id)
                     & (Submission.date == subquery.c.max_date),
                 )
                 .order_by(Assignment.id)
@@ -141,7 +140,7 @@ class SubmissionLectureHandler(GraderBaseHandler):
             submissions = (
                 submissions_query.join(
                     subquery,
-                    (Submission.username == subquery.c.username)
+                    (Submission.user_id == subquery.c.user_id)
                     & (Submission.score == subquery.c.max_score),
                 )
                 .order_by(Submission.id)
@@ -221,16 +220,13 @@ class SubmissionHandler(GraderBaseHandler):
         if submission_filter not in ["none", "latest", "best"]:
             raise HTTPError(
                 HTTPStatus.BAD_REQUEST,
-                reason="Filter parameter \
-            has to be either 'none', 'latest' or 'best'",
+                reason="Filter parameter has to be either 'none', 'latest' or 'best'",
             )
         instr_version = self.get_argument("instructor-version", None) == "true"
         response_format = self.get_argument("format", "json")
         if response_format not in ["json", "csv"]:
             raise HTTPError(
-                HTTPStatus.BAD_REQUEST,
-                reason="Response format \
-            can either be 'json' or 'csv'",
+                HTTPStatus.BAD_REQUEST, reason="Response format can either be 'json' or 'csv'"
             )
 
         # check required scopes for instructor version
@@ -241,26 +237,21 @@ class SubmissionHandler(GraderBaseHandler):
         self.validate_assignment(lecture_id, assignment_id)
 
         # get list of submissions based on arguments
-        username = None if instr_version else role.username
+        user_id = None if instr_version else role.user_id
         if submission_filter == "latest":
-            submissions = self.get_latest_submissions(assignment_id, username=username)
+            submissions = self.get_latest_submissions(assignment_id, user_id=user_id)
         elif submission_filter == "best":
-            submissions = self.get_best_submissions(assignment_id, username=username)
+            submissions = self.get_best_submissions(assignment_id, user_id=user_id)
         else:
             query = self.session.query(Submission).filter(
                 Submission.assignid == assignment_id, Submission.deleted == DeleteState.active
             )
-            if username:
-                query = query.filter(Submission.username == username)
+            if user_id:
+                query = query.filter(Submission.user_id == user_id)
             submissions = query.order_by(Submission.id).all()
 
         if response_format == "csv":
-            self.set_header("Content-Type", "text/csv")
-            for i, s in enumerate(submissions):
-                d = s.model.to_dict()
-                if i == 0:
-                    self.write(",".join((k for k in d.keys() if k != "logs")) + "\n")
-                self.write(",".join((str(v) for k, v in d.items() if k != "logs")) + "\n")
+            self._write_csv(submissions)
         else:
             if not instr_version:
                 submissions = remove_points_from_submission(submissions)
@@ -268,6 +259,14 @@ class SubmissionHandler(GraderBaseHandler):
             self.write_json(submissions)
         self.session.close()
 
+    def _write_csv(self, submissions):
+        self.set_header("Content-Type", "text/csv")
+        for i, s in enumerate(submissions):
+            d = s.model.to_dict()
+            if i == 0:
+                self.write(",".join((k for k in d.keys() if k != "logs")) + "\n")
+            self.write(",".join((str(v) for k, v in d.items() if k != "logs")) + "\n")
+
     @authorize([Scope.student, Scope.tutor, Scope.instructor])
     async def post(self, lecture_id: int, assignment_id: int):
         """Create submission based on commit hash.
@@ -293,7 +292,7 @@ class SubmissionHandler(GraderBaseHandler):
         if assignment.status == "complete":
             raise HTTPError(HTTPStatus.BAD_REQUEST, reason="Cannot submit completed assignment!")
         if role.role == Scope.student and assignment.status != "released":
-            raise HTTPError(HTTPStatus.NOT_FOUND)
+            raise HTTPError(HTTPStatus.NOT_FOUND, "Assignment not found")
         # set utc time
         submission_ts = datetime.datetime.now(datetime.timezone.utc)
         # use implicit utc time to compare with database objects
@@ -303,13 +302,10 @@ class SubmissionHandler(GraderBaseHandler):
         if assignment.settings.deadline is not None:
             score_scaling = self.calculate_late_submission_scaling(assignment, submission_ts, role)
 
-        if assignment.settings.max_submissions:
+        if assignment.settings.max_submissions and role.role < Scope.tutor:
             submissions = assignment.submissions
-            usersubmissions = [s for s in submissions if s.username == role.username]
-            if (
-                len(usersubmissions) >= assignment.settings.max_submissions
-                and role.role < Scope.tutor
-            ):
+            user_submissions = [s for s in submissions if s.user_id == role.user_id]
+            if len(user_submissions) >= assignment.settings.max_submissions:
                 raise HTTPError(
                     HTTPStatus.CONFLICT, reason="Maximum number of submissions reached!"
                 )
@@ -317,36 +313,48 @@ class SubmissionHandler(GraderBaseHandler):
         submission = Submission()
         submission.assignid = assignment.id
         submission.date = submission_ts
-        submission.username = self.user.name
         submission.score_scaling = score_scaling
 
-        git_repo_path = self.construct_git_dir(
-            repo_type=GitRepoType.USER, lecture=assignment.lecture, assignment=assignment
-        )
+        username = body.get("username")
+        if username is not None and role.role >= Scope.tutor:
+            # Instructor/tutor creates a submission for a student:
+            s_user = self.session.query(User).filter(User.name == username).one_or_none()
+            if s_user is None:
+                raise HTTPError(HTTPStatus.NOT_FOUND, reason=f"User {username} not found")
+            s_user_role = self.session.get(Role, (s_user.id, lecture_id))
+            if s_user_role is None:
+                raise HTTPError(
+                    HTTPStatus.NOT_FOUND,
+                    reason=f"User {username} does not take part in this lecture",
+                )
+            submission.user_id = s_user.id
+        else:
+            # A user creates a submission for themselves.
+            submission.user_id = self.user.id
 
-        try:
-            # Commit hash "0"*40 is used to differentiate between submissions created by
-            # instructors for students and normal submissions by any user.
-            # In this case submissions for the student might not exist, so we cannot reference
-            # a non-existing commit_hash. When submission is set to edited, autograder uses edit
-            # repository, so we don't need the commit_hash of the submission.
-            if commit_hash != "0" * 40:
-                if not os.path.exists(git_repo_path):
-                    raise HTTPError(
-                        HTTPStatus.UNPROCESSABLE_ENTITY, reason="User git repository not found"
-                    )
+            git_repo_path = self.construct_git_dir(
+                repo_type=GitRepoType.USER, lecture=assignment.lecture, assignment=assignment
+            )
+
+            # If no submissions for the student exists, we cannot reference a non-existing
+            # commit_hash.
+            if not os.path.exists(git_repo_path):
+                raise HTTPError(
+                    HTTPStatus.UNPROCESSABLE_ENTITY, reason="User git repository not found"
+                )
+            try:
                 subprocess.run(
                     ["git", "branch", "main", "--contains", commit_hash],
                     cwd=git_repo_path,
                     capture_output=True,
                 )
-        except subprocess.CalledProcessError:
-            raise HTTPError(HTTPStatus.NOT_FOUND, reason="Commit not found")
+            except subprocess.CalledProcessError:
+                raise HTTPError(HTTPStatus.NOT_FOUND, reason="Commit not found")
 
         submission.commit_hash = commit_hash
-        submission.auto_status = "not_graded"
-        submission.manual_status = "not_graded"
-        submission.feedback_status = "not_generated"
+        submission.auto_status = AutoStatus.NOT_GRADED
+        submission.manual_status = ManualStatus.NOT_GRADED
+        submission.feedback_status = FeedbackStatus.NOT_GENERATED
 
         automatic_grading = assignment.settings.autograde_type
 
@@ -355,25 +363,31 @@ class SubmissionHandler(GraderBaseHandler):
         self.set_status(HTTPStatus.CREATED)
         self.write_json(submission)
 
-        # If the assignment has automatic grading or fully
-        # automatic grading perform necessary operations
-        if automatic_grading in ["auto", "full_auto"]:
-            submission.auto_status = "pending"
+        # If the assignment has automatic or fully automatic grading, run the necessary tasks.
+        # The autograding is not performed when an instructor creates a new submission for a student.
+        # Note: A specific commit hash is used to differentiate between submissions created by
+        # instructors for students and normal submissions by any user.
+        if (
+            automatic_grading in ["auto", "full_auto"]
+            and commit_hash != INSTRUCTOR_SUBMISSION_COMMIT_CASH
+        ):
+            submission.auto_status = AutoStatus.PENDING
             self.session.commit()
             self.set_status(HTTPStatus.ACCEPTED)
 
             if automatic_grading == "full_auto":
-                submission.feedback_status = "generating"
+                submission.feedback_status = FeedbackStatus.GENERATING
                 self.session.commit()
 
-                # use immutable signature: https://docs.celeryq.dev/en/stable/reference/celery.app.task.html#celery.app.task.Task.si
+                # use immutable signature:
+                # https://docs.celeryq.dev/en/stable/reference/celery.app.task.html#celery.app.task.Task.si
                 grading_chain = chain(
                     autograde_task.si(lecture_id, assignment_id, submission.id),
                     generate_feedback_task.si(lecture_id, assignment_id, submission.id),
                     lti_sync_task.si(
                         lecture.serialize(),
                         assignment.serialize(),
-                        [submission.serialize()],
+                        [submission.serialize_with_user()],
                         feedback_sync=True,
                     ),
                 )
@@ -449,10 +463,7 @@ class SubmissionObjectHandler(GraderBaseHandler):
             lecture_id, assignment_id, submission_id
         )
         submission = self.get_submission(lecture_id, assignment_id, submission_id)
-        if (
-            self.get_role(lecture_id).role == Scope.student
-            and submission.username != self.user.name
-        ):
+        if self.get_role(lecture_id).role == Scope.student and submission.user_id != self.user.id:
             raise HTTPError(HTTPStatus.NOT_FOUND)
         self.write_json(submission)
 
@@ -478,7 +489,7 @@ class SubmissionObjectHandler(GraderBaseHandler):
 
         role = self.get_role(lecture_id)
         if role.role >= Scope.instructor:
-            sub.username = sub_model.username
+            sub.user_id = sub_model.user_id
         sub.auto_status = sub_model.auto_status
         sub.manual_status = sub_model.manual_status
         sub.edited = sub_model.edited
@@ -491,8 +502,9 @@ class SubmissionObjectHandler(GraderBaseHandler):
         self.session.commit()
         self.write_json(sub)
 
-    def delete(self, lecture_id: int, assignment_id: int, submission_id: int):
-        """Soft-Deletes a specific submission.
+    @authorize([Scope.student, Scope.tutor, Scope.instructor])
+    async def delete(self, lecture_id: int, assignment_id: int, submission_id: int):
+        """Soft-deletes a specific submission.
 
         :param lecture_id: id of the lecture
         :type lecture_id: int
@@ -500,7 +512,9 @@ class SubmissionObjectHandler(GraderBaseHandler):
         :type assignment_id: int
         :param submission_id: id of the submission
         :type submission_id: int
-        :raises HTTPError: if submission can't be deleted due to it having feedback
+        :raises HTTPError: if submission has feedback, or the deadline has passed,
+            or it has already been (soft-)deleted, or it belongs to another student,
+            or it was not found in the given lecture and assignment.
         """
         lecture_id, assignment_id, submission_id = parse_ids(
             lecture_id, assignment_id, submission_id
@@ -508,40 +522,32 @@ class SubmissionObjectHandler(GraderBaseHandler):
         self.validate_parameters()
         submission = self.get_submission(lecture_id, assignment_id, submission_id)
 
-        if submission is not None:
-            if submission.feedback_status != "not_generated":
-                raise HTTPError(
-                    HTTPStatus.FORBIDDEN, reason="Only submissions without feedback can be deleted."
-                )
-            # if assignment has deadline
-            if submission.assignment.settings.deadline:
-                # if assignment's deadline has passed
-                if submission.assignment.settings.deadline < datetime.datetime.now(
-                    datetime.timezone.utc
-                ):
-                    raise HTTPError(
-                        HTTPStatus.FORBIDDEN,
-                        reason="Submission can't be deleted, due date of assigment has passed.",
-                    )
-            else:
-                previously_deleted = (
-                    self.session.query(Submission)
-                    .filter(
-                        Submission.id == submission_id,
-                        Submission.assignid == assignment_id,
-                        Submission.deleted == DeleteState.deleted,
-                    )
-                    .one_or_none()
-                )
-                if previously_deleted is not None:
-                    self.session.delete(previously_deleted)
-                    self.session.commit()
-
-                submission.deleted = DeleteState.deleted
-                self.session.commit()
-        else:
+        if submission is None:
             raise HTTPError(HTTPStatus.NOT_FOUND, reason="Submission to delete not found.")
 
+        # Do not allow students to delete other users' submissions
+        if self.get_role(lecture_id).role < Scope.instructor and submission.user_id != self.user.id:
+            raise HTTPError(HTTPStatus.NOT_FOUND, reason="Submission to delete not found.")
+
+        if submission.feedback_status != FeedbackStatus.NOT_GENERATED:
+            raise HTTPError(
+                HTTPStatus.FORBIDDEN, reason="Only submissions without feedback can be deleted."
+            )
+
+        # if assignment has deadline and it has already passed
+        if (
+            submission.assignment.settings.deadline
+            and submission.assignment.settings.deadline
+            < datetime.datetime.now(datetime.timezone.utc)
+        ):
+            raise HTTPError(
+                HTTPStatus.FORBIDDEN,
+                reason="Submission can't be deleted, due date of assigment has passed.",
+            )
+
+        submission.deleted = DeleteState.deleted
+        self.session.commit()
+
 
 @register_handler(
     path=r"\/api\/lectures\/(?P<lecture_id>\d*)\/assignments\/"
@@ -568,7 +574,7 @@ class SubmissionLogsHandler(GraderBaseHandler):
         if logs is not None:
             self.write_json(logs.logs)
         else:
-            raise HTTPError(HTTPStatus.NOT_FOUND, reason="Properties of submission were not found")
+            raise HTTPError(HTTPStatus.NOT_FOUND, reason="Logs of submission were not found")
 
 
 @register_handler(
@@ -632,24 +638,23 @@ class SubmissionPropertiesHandler(GraderBaseHandler):
 
         try:
             gradebook = GradeBookModel.from_dict(json.loads(properties_string))
-
-            score = gradebook.score
-            submission.grading_score = score
-            submission.score = submission.score_scaling * score
-
         except Exception as e:
             self.log.info(e)
             raise HTTPError(HTTPStatus.BAD_REQUEST, reason="Cannot parse properties file!")
 
+        score = gradebook.score
+        submission.grading_score = score
+        submission.score = submission.score_scaling * score
+
         properties = SubmissionProperties(properties=properties_string, sub_id=submission.id)
 
         self.session.merge(properties)
 
-        if submission.feedback_status == "generated":
-            submission.feedback_status = "feedback_outdated"
+        if submission.feedback_status == FeedbackStatus.GENERATED:
+            submission.feedback_status = FeedbackStatus.FEEDBACK_OUTDATED
 
-        if submission.manual_status == "manually_graded":
-            submission.manually_graded = "being_edited"
+        if submission.manual_status == ManualStatus.MANUALLY_GRADED:
+            submission.manually_graded = ManualStatus.BEING_EDITED
 
         self.session.commit()
         self.write_json(submission)
@@ -672,7 +677,7 @@ class SubmissionPropertiesHandler(GraderBaseHandler):
 class SubmissionEditHandler(GraderBaseHandler):
     @authorize([Scope.tutor, Scope.instructor])
     async def put(self, lecture_id: int, assignment_id: int, submission_id: int):
-        """Create or overwrites the repository which stores changes of
+        """Creates or overwrites (resets) the repository which stores changes of
         submissions files
         :param lecture_id: lecture id
         :param assignment_id: assignment id
@@ -685,24 +690,31 @@ class SubmissionEditHandler(GraderBaseHandler):
         self.validate_parameters()
 
         submission = self.get_submission(lecture_id, assignment_id, submission_id)
+        if submission.commit_hash == INSTRUCTOR_SUBMISSION_COMMIT_CASH:
+            raise HTTPError(
+                HTTPStatus.BAD_REQUEST,
+                reason="This repo cannot be edited or reset, because it was created by instructor",
+            )
+
         assignment = submission.assignment
         lecture = assignment.lecture
 
-        # Path to repository which will store edited submission files
-        git_repo_path = os.path.normpath(
-            os.path.join(self.gitbase, lecture.code, str(assignment.id), "edit", str(submission_id))
+        # Path to the (bare!) repository which will store edited submission files
+        git_repo_path = self.construct_git_dir(
+            repo_type=GitRepoType.EDIT,
+            lecture=lecture,
+            assignment=assignment,
+            submission=submission,
         )
-        if not git_repo_path.startswith(self.gitbase):
-            raise HTTPError(HTTPStatus.BAD_REQUEST, reason="Invalid repository path.")
 
         # Path to repository of student which contains the submitted files
-        submission_repo_path = os.path.normpath(
-            os.path.join(
-                self.gitbase, lecture.code, str(assignment.id), "user", submission.username
-            )
+        submission_repo_path = os.path.join(
+            self.gitbase, lecture.code, str(assignment.id), "user", submission.user.name
         )
-        if not submission_repo_path.startswith(self.gitbase):
-            raise HTTPError(HTTPStatus.BAD_REQUEST, reason="Invalid submission repository path.")
+        if not os.path.exists(submission_repo_path):
+            raise HTTPError(
+                HTTPStatus.BAD_REQUEST, reason="The user submission repository does not exist"
+            )
 
         if os.path.exists(git_repo_path):
             shutil.rmtree(git_repo_path)
@@ -711,10 +723,11 @@ class SubmissionEditHandler(GraderBaseHandler):
         if not os.path.exists(git_repo_path):
             os.makedirs(git_repo_path, exist_ok=True)
 
-        await self._run_command_async(["git", "init", "--bare"], git_repo_path)
+        await self._run_command_async(
+            ["git", "init", "--bare", "--initial-branch=main"], git_repo_path
+        )
 
-        # Create temporary paths to copy the submission
-        # files in the edit repository
+        # Create temporary paths to copy the submission files in the edit repository
         tmp_path = os.path.join(
             self.application.grader_service_dir,
             "tmp",
@@ -734,7 +747,7 @@ class SubmissionEditHandler(GraderBaseHandler):
         os.makedirs(tmp_input_path, exist_ok=True)
 
         # Init local repository
-        await self._run_command_async(["git", "init"], tmp_input_path)
+        await self._run_command_async(["git", "init", "--initial-branch=main"], tmp_input_path)
 
         # Pull user repository
         await self._run_command_async(
@@ -750,7 +763,7 @@ class SubmissionEditHandler(GraderBaseHandler):
         shutil.copytree(tmp_input_path, tmp_output_path, ignore=shutil.ignore_patterns(".git"))
 
         # Init local repository
-        await self._run_command_async(["git", "init"], tmp_output_path)
+        await self._run_command_async(["git", "init", "--initial-branch=main"], tmp_output_path)
 
         # Add edit remote
         await self._run_command_async(
@@ -833,7 +846,7 @@ class LtiSyncHandler(GraderBaseHandler):
                     self.session.query(Submission)
                     .filter(
                         Submission.id.in_(submission_ids),
-                        Submission.auto_status == "automatically_graded",
+                        Submission.auto_status == AutoStatus.AUTOMATICALLY_GRADED,
                         Submission.assignid == assignment_id,
                     )
                     .all()
@@ -852,7 +865,8 @@ class LtiSyncHandler(GraderBaseHandler):
         lti_plugin = LTISyncGrades.instance()
         lecture_model = lecture.serialize()
         assignment_model = assignment.serialize()
-        submissions_model = [sub.serialize() for sub in submissions if isinstance(sub, Submission)]
+        submissions_model = [sub.serialize_with_user() for sub in submissions]
+
         # check if the lti plugin is enabled
         if lti_plugin.check_if_lti_enabled(
             lecture_model, assignment_model, submissions_model, feedback_sync=False
@@ -866,9 +880,9 @@ class LtiSyncHandler(GraderBaseHandler):
                 raise HTTPError(HTTPStatus.INTERNAL_SERVER_ERROR, reason=err_msg)
             except Exception as e:
                 self.log.error("Could not sync grades: " + str(e))
-                raise HTTPError(500, reason="An unexpected error occured.")
+                raise HTTPError(500, reason="An unexpected error occurred.")
         else:
-            raise HTTPError(403, reason="LTI plugin is not enabled by administator.")
+            raise HTTPError(403, reason="LTI plugin is not enabled by administrator.")
 
 
 @register_handler(
@@ -895,7 +909,7 @@ class SubmissionCountHandler(GraderBaseHandler):
 
         usersubmissions_count = (
             self.session.query(Submission)
-            .filter(Submission.assignid == assignment_id, Submission.username == role.username)
+            .filter(Submission.assignid == assignment_id, Submission.user_id == role.user_id)
             .count()
         )
 
