@@ -149,26 +149,38 @@ class GraderService(config.Application):
     oauth_token_expires_in = int(1 * 24 * 3600)
 
     load_roles = Dict(
-        Dict(),
+        List(),
         help="""
-        Dict of `{'<lecture-code>': {members: list<str>, role: str}}`  to load at startup.
+        Dict of `'<lecture-code>': List[{'members': List[Union[str, Dict[str, Optional[str]]]], 'role': str}]` entries to load at startup.
+
+        Each lecture code maps to a list of role definitions. Each role definition must include:
+          - 'role': the role name (e.g. 'student', 'tutor', 'instructor')
+          - 'members': either
+              * a list of username strings (legacy format), e.g. ['alice', 'bob']
+              OR
+              * a list of user objects, each with:
+                  - 'username' (required): the unique username
+                  - 'display_name' (optional): the user's display name; if omitted, defaults to the username
 
         Example::
 
             c.GraderService.load_roles = {
-                'lecture1': {
-                    'members': ['student1', 'student2'],
+                'lecture1': [
+                    {
+                    'members': ['student1', 'student2'],  # legacy simple list
                     'role': 'student'
-                    }
-                },
-                'lecture1': {
-                    'members': ['instructor1', 'instructor2'],
+                    },
+                    {
+                    'members': [
+                        {'username': 'instructor1', 'display_name': 'Instructor 1'},
+                        {'username': 'instructor2'}
+                    ],
                     'role': 'instructor'
                     }
-                },
+                ],
             }
 
-        This will replace all existing roles.
+
         """,
     ).tag(config=True)
 
@@ -176,7 +188,9 @@ class GraderService(config.Application):
         Dict(),
         default_value=[],
         help="""
-        List of OAuth clients `[{'client_id': '<client_id>', 'client_secret': '<client_secret>', 'redirect_uri': '<redirect_uri>'}]` to register for the provider.
+        List of OAuth clients 
+        `[{'client_id': '<client_id>', 'client_secret': '<client_secret>', 'redirect_uri': '<redirect_uri>'}]`
+        to register for the provider.
         
         Example::
             
@@ -293,10 +307,7 @@ class GraderService(config.Application):
                 f"The directory to write the config file has to exist. {config_file_dir} not found"
             )
         if os.path.isfile(os.path.abspath(self.config_file)):
-            self.exit(
-                f"Config file {os.path.abspath(self.config_file)} \
-                already exists!"
-            )
+            self.exit(f"Config file {os.path.abspath(self.config_file)} already exists!")
 
         members = inspect.getmembers(
             sys.modules[__name__], lambda x: inspect.isclass(x) and issubclass(x, HasTraits)
@@ -358,43 +369,53 @@ class GraderService(config.Application):
         """Load predefined groups into the database"""
         with self.session_maker() as db:
             users_loaded = set()
-            for lecture_code, role_dict in self.load_roles.items():
-                role = role_dict.get("role")
-                users = role_dict.get("members", [])
-                lecture = db.query(Lecture).filter(Lecture.code == lecture_code).one_or_none()
-                # create lecture if no lecture with that name exists yet
-                # (code is set in create)
-                if lecture is None:
-                    self.log.info(f"Adding new lecture with lecture_code {lecture_code}")
-                    lecture = Lecture()
-                    lecture.code = lecture_code
-                    lecture.name = lecture_code
-                    lecture.state = LectureState.active
-                    lecture.deleted = DeleteState.active
-                    db.add(lecture)
-                db.commit()
+            for lecture_code in self.load_roles.keys():
+                role_list = self.load_roles[lecture_code]
+                for role_dict in role_list:
+                    role = role_dict.get("role")
+                    users = role_dict.get("members", [])
+                    lecture = db.query(Lecture).filter(Lecture.code == lecture_code).one_or_none()
+                    # create lecture if no lecture with that name exists yet
+                    # (code is set in create)
+                    if lecture is None:
+                        self.log.info(f"Adding new lecture with lecture_code {lecture_code}")
+                        lecture = Lecture()
+                        lecture.code = lecture_code
+                        lecture.name = lecture_code
+                        lecture.state = LectureState.active
+                        lecture.deleted = DeleteState.active
+                        db.add(lecture)
+                    db.commit()
 
-                for username in users:
-                    user = db.query(User).filter(User.name == username).one_or_none()
-                    if user is None:
-                        self.log.info(f"Adding new user with username {username}")
-                        user = User()
-                        user.name = username
-                        user.display_name = username
-                        db.add(user)
-                        db.commit()
+                    for user_entry in users:
+                        if isinstance(user_entry, dict):
+                            username = user_entry.get("username")
+                            display_name = user_entry.get("display_name") or username
+                        else:
+                            # fallback for old format (just a username string)
+                            username = user_entry
+                            display_name = username
 
-                    # delete all roles of users the first time a new role is added for the user
-                    if user.name not in users_loaded:
-                        db.query(Role).filter(Role.username == user.name).delete()
-                        users_loaded.add(user.name)
+                        user = db.query(User).filter(User.name == username).one_or_none()
+                        if user is None:
+                            self.log.info(f"Adding new user with username {username} and display name {display_name}")
+                            user = User()
+                            user.name = username
+                            user.display_name = display_name
+                            db.add(user)
+                            db.commit()
 
-                    try:
-                        db.add(Role(username=user.name, lectid=lecture.id, role=Scope[role]))
-                    except KeyError:
-                        self.log.error(f"Invalid role name: {role}")
-                        raise ValueError(f"Invalid role name: {role}")
-                db.commit()
+                        # delete all roles of users the first time a new role is added for the user
+                        if user.name not in users_loaded:
+                            db.query(Role).filter(Role.user_id == user.id).delete()
+                            users_loaded.add(user.name)
+
+                        try:
+                            db.add(Role(user_id=user.id, lectid=lecture.id, role=Scope[role]))
+                        except KeyError:
+                            self.log.error(f"Invalid role name: {role}")
+                            raise ValueError(f"Invalid role name: {role}")
+                    db.commit()
 
     async def start(self):
         self.log.info(f"Config File: {os.path.abspath(self.config_file)}")
@@ -419,7 +440,8 @@ class GraderService(config.Application):
         auth_handlers = self.authenticator.get_handlers(self.base_url_path)
         handlers.extend(auth_handlers)
         self.log.info(
-            f"Registered authentication handlers for {self.authenticator.__class__.__name__}: {[n for n, _ in auth_handlers]}"
+            f"Registered authentication handlers for {self.authenticator.__class__.__name__}: "
+            f"{[n for n, _ in auth_handlers]}"
         )
 
         oauth_provider_handlers = oauth_handlers.get_oauth_default_handlers(self.base_url_path)
